import React, { useEffect, useState } from 'react'
import { ScrollView } from 'react-native'

import Default from '@holi/core/components/ScrollHeaderLayout/Default'
import { RefreshableScrollHeaderLayoutProps } from '@holi/core/components/ScrollHeaderLayout/types'
import HoliLoader from '@holi/ui/components/molecules/HoliLoader'
import { vars } from '@holi/ui/styles/globalVars'

const PULL_DEVIDER = 2

const Refreshable = React.forwardRef<ScrollView, RefreshableScrollHeaderLayoutProps>(
  (
    {
      refreshableProps: {
        Loader,
        isLoading,
        onRefresh,
        refreshHeight = +vars.topBarHeight + +vars.loaderHeight,
        loaderTopOffset = +vars.loaderHeight,
      },
      ...defaultLayoutProps
    },
    ref
  ) => {
    const defaultLoader = () => <HoliLoader style={{ top: loaderTopOffset }} />
    Loader = defaultLoader || Loader

    const [startPoint, setStartPoint] = useState(0)
    const [pullChange, setPullChange] = useState(0)

    const pullStart = (event: TouchEvent) => {
      const { screenY } = event.targetTouches[0]
      setStartPoint(screenY)
    }

    const pull = (event: TouchEvent) => {
      // get the current user touch event data
      const touch = event.targetTouches[0]
      // get the touch position on the screen's Y axis
      const { screenY } = touch
      // The length of the pull
      // if the start touch position is lesser than the current touch position, calculate the difference, which gives the `pullLength`
      // This tells us how much the user has pulled
      const pullLength = startPoint < screenY ? Math.abs(screenY - startPoint) : 0
      setPullChange(pullLength / PULL_DEVIDER)
    }

    const endPull = () => {
      if (pullChange >= refreshHeight) {
        onRefresh()
      }

      setStartPoint(0)
      setPullChange(0)
    }

    useEffect(() => {
      window.addEventListener('touchstart', pullStart)
      window.addEventListener('touchmove', pull)
      window.addEventListener('touchend', endPull)
      return () => {
        window.removeEventListener('touchstart', pullStart)
        window.removeEventListener('touchmove', pull)
        window.removeEventListener('touchend', endPull)
      }
    })

    useEffect(() => {
      if (!isLoading) {
        setPullChange(0)
      }
    }, [isLoading])

    return (
      <div
        className="refreshable-wrapper"
        style={{
          overscrollBehaviorY: 'contain',
          marginTop: isLoading ? refreshHeight : pullChange,
          // animate only on endPull
          transition: !startPoint ? 'all .3s ease-out' : '',
        }}
      >
        {(!!startPoint || isLoading) && (
          <div
            style={{
              position: 'absolute',
              top: 0,
              left: 0,
              right: 0,
              height: refreshHeight,
              transform: `translateY(${(pullChange / refreshHeight) * 200})`,
            }}
          >
            <Loader />
          </div>
        )}
        <Default {...defaultLayoutProps} ref={ref} />
      </div>
    )
  }
)

Refreshable.displayName = 'ScrollHeaderLayout.Refreshable'

export default Refreshable
